/*
m_scene_speakers - A very simplistc emulation of a pair of speakers

DESCRIPTION
A very simplistic emulation of a pair of speakers with low and high cut,
rolloff at the edges and a size parameter.
NOTE: The volume is adjusted internally, but it may still overdrive!

SYNTAX
aoutl, aoutr m_scene_speakers ainl, ainr, isize, ilowcut, ihighcut, irolloff

INITIALIZATION
isze - Size of the speaker in meters, basically the width of a cube-like speaker
ilowcut - Lowcut frequency in Hertz
ihighcut - Highcut frequency in Hertz
irolloff - Rolloff characteristic for low- and highcut.
	<1: 1 pole filters, i.e. 6dB/octave
	>=1 2pole filter, i.e. 12dB/octave with Q adjustment between 1 and 2

PERFORMANCE
ainl, ainr - Input audio signals to be processed

CREDITS
Author: Jeanette C.
*/

opcode m_scene_speakers, aa, aaiiii
	ainl, ainr, isize, ilow, ihigh, irolloff xin

	; Set up basic variables
	alpfl init 0
	ahpfl init 0
	alpfr init 0
	ahpfr init 0

	; Apply low and high cut
	if irolloff < 1 then
		alpfl zdf_1pole ainl, ihigh, 0, 0
		ahpfl zdf_1pole alpfl, ilow, 1, 0
		alpfr zdf_1pole ainr, ihigh, 0, 0
		ahpfr zdf_1pole alpfr, ilow, 1, 0
		ivol init .9
	else
		if irolloff <= 2 then
			iq = .5 + (1.5 * irolloff)
		else
			iq = 2
		endif
		alpfl zdf_2pole ainl, ihigh, iq, 0
		ahpfl zdf_2pole alpfl, ilow, iq, 1
		alpfr zdf_2pole ainr, ihigh, iq, 0
		ahpfr zdf_2pole alpfr, ilow, iq, 1
		ivol init .7
	endif

	; apply speaker's resonance
	aoutl wguide1 ahpfl, (340 / isize), ihigh/2, .1
	aoutr wguide1 ahpfr, (340 / isize), ihigh/2, .1

	xout (aoutl * ivol), (aoutr * ivol)
endop
